//	74.442	Advanced Design & Analysis of Algorithms
//	Spring 1998
//	Dr. Bill Kocay
//
//	Hamish Carr
//	Project:	Toroidal Surface Geometries
//
//

#include "TorusApplication.h"

TorusApplication::TorusApplication()										//	constructor
	: BApplication("application/x-torus")									//	inherited constructor
	{
	itsOpenFilePanel = new BFilePanel;										//	construct an open panel with default arguments
	itsSettingsWindow = new TorusSettingsWindow();							//	create a settings window
	NewTorusWindow();														//	call the new window routine
	} // end of TorusApplication constructor

void TorusApplication::NewTorusWindow()										//	new window
	{
	if (itsSettingsWindow->IsHidden())										//	if the settings window is hidden
		itsSettingsWindow->Show();											//	show it
	} // end of NewWindow()

void TorusApplication::MessageReceived(BMessage *theMessage)				//	deals with application messages
	{
	switch(theMessage->what)												//	yer basic case statement
		{
		case AboutTorusItem:												//	the user has asked for the about box
			BAlert *aboutBox = new BAlert("Torus Settings Help", TORUS_ABOUT_MESSAGE, "Done", NULL, NULL, 
							B_WIDTH_FROM_LABEL,B_STOP_ALERT);				//	set the alert up
			aboutBox->Go();													//	run it & wait for it to return
			break;
		case NewTorusItem:													//	user has asked for another torus
			NewTorusWindow();												//	open a new torus window
			break;
		case OpenTorusItem:													//	user has asked to open a file on disk
			itsOpenFilePanel->Show();										//	show the open file panel
			break;
		case B_CANCEL:														//	the file-panel was closed
			long nWindows;													//	how many windows the app has
			nWindows = CountWindows();										//	find out how many
			if (nWindows <= 2)												//	i.e. this is the last window other than the new torus panel
				PostMessage(B_QUIT_REQUESTED);								//	tell the app we would like to quit
			break;
		default:
			BApplication::MessageReceived(theMessage);						//	pass it on to the BWindow class
			break;
		} // end of switch statement
	} // end of MessageReceived()
	
void TorusApplication::RefsReceived(BMessage *theMessage)					//	deals with file opening
	{
	uint32 type; 															//	type of ref received
  	int32 count; 															//	how many refs were received
  	entry_ref ref; 															//	the next ref
	off_t fileSize;															//	the size of the file (in bytes)
	char *inBuffer;															//	a text buffer for reading the file in
	ssize_t bytesRead;														//	how many bytes were actually read in
	long i;																	//	loop index
	status_t errorCode;														//	return value of functions
    BFile theFile;															//	pointer to the file in question 
	TorusWindow *aTorusWindow;												//	a window for the torus
 	BEntry theEntry;														//	the location of the file
	BEntry parentEntry;														//	the directory where the file can be found
	entry_ref parentRef;													//	the corresponding entry_ref structure
	BMessage *location;														//	location of the file, stored as a message 
	char fileName[B_FILE_NAME_LENGTH];										//	name of the file 
	    
	theMessage->GetInfo("refs", &type, &count); 							//	find out how many refs of which type
	if (type != B_REF_TYPE)													//	if "refs" is something else, bail out 
		return; 
	for (i = --count; i >= 0; i--)											//	start with the last ref, and work down 
	  	{
	    errorCode = theMessage->FindRef("refs", i, &ref);					//	try to retrieve the next ref.
		if (errorCode != B_OK) continue;									//	if it doesn't exist, try the next one
	  	errorCode = theFile.SetTo(&ref, O_RDWR);							//	try to open the file with R/W permission
		if (errorCode != B_OK) continue;									//	if we failed to set the file to the ref, try next one
		errorCode = theEntry.SetTo(&ref);									//	try and get the file's Entry
		if (errorCode != B_OK) continue;									//	if it doesn't exist, try the next one
		errorCode = theEntry.GetParent(&parentEntry);						//	retrieve the parent directory
		if (errorCode != B_OK) continue;									//	if it doesn't exist, try the next one
		errorCode = parentEntry.GetRef(&parentRef);							//	retrieve the parent reference
		if (errorCode != B_OK) continue;									//	if it doesn't exist, try the next one
		errorCode = theFile.GetSize(&fileSize);								//	try to find out how big the file is
		if (errorCode == B_NO_ERROR)										//	if it works, keep going
			{
			if (fileSize == 0)												//	if the file is empty
				DoAlert("Attempt to read empty file.\n");					//	tell the user
			else															//	i.e. file is non-empty
				{
				inBuffer = new char[fileSize+1];							//	allocate space for the contents of the file
				if (inBuffer != NULL)										//	if the allocate succeeded
					{
					bytesRead = theFile.Read(inBuffer, fileSize);			//	read the actual file in
					if (bytesRead == fileSize)								//	if we read the right number of bytes
						{
						inBuffer[fileSize] = '\0';							// 	null-terminate it
						location = new BMessage(B_SAVE_REQUESTED);			//	construct the save message
						location->AddRef("directory", &parentRef);			//	add the directory to it
						theEntry.GetName(fileName);							//	retrieve the name
						location->AddString("name", fileName);				//	add it to the message				
						aTorusWindow = new TorusWindow(defaultTorusWindowRect, inBuffer, location);
																			//	create the torus in it's window
						aTorusWindow->Show();								//	and show it
						} // end of successful read	
					else delete inBuffer;									//	release the storage
					} // end of test for allocation of buffer					
				} // end of test for non-zero file size
         	} // end of test to retrieve file size	
	   	theFile.Unset();  													//	dissociate ref from thefile
		} // end of loop through all refs 
	} // end of RefsReceived()