//	74.442	Advanced Design & Analysis of Algorithms
//	Spring 1998
//	Dr. Bill Kocay
//
//	Hamish Carr
//	Project:	Toroidal Surface Geometries
//
//

#include "Torus.h"

void Torus::Rotate(Basis &rotation)											//	apply transformation matrix
	{
	Vector *theVector;														//	pointer walker
	Vector *stopVector;														//	sentinel value
	BPoint *thePoint;														//	pointer walker
	Tile *theTile;															//	tile walker
	float zScale;															//	amount to scale, based on z
	float r;																//	used to calculate whether we are
																			//	inside torus
	Vector Origin;															//	the origin in torus-basis coordinates
	
	stopVector = theGridVectors + nTiles;									//	set sentinel
	theVector = theGridVectors;												//	start our walker
	thePoint = theGridPoints;												//	make that walker*s*
	theTile = theTiles;
	
	torusOrigin = rotation*torusOrigin;										//	move the origin of the torus
	torusBasis.i = rotation*torusBasis.i;									//	rotate each basis vector
	torusBasis.j = rotation*torusBasis.j;
	torusBasis.k = rotation*torusBasis.k;

	Origin = torusBasis*torusOrigin;	

	r = sqrt(Origin.x*Origin.x+Origin.y*Origin.y);							//	calculate sqrt (x2 + y2)
	r = r - ringRadius;														//	subtract the ring radius
	r = sqrt(r*r + Origin.z*Origin.z);										//	finish the formula	
	insideTorus = (r <= radius);											//	determine if we're inside the torus

	for (; theVector < stopVector;)											//	walk down three arrays in parallel
		{
		*theVector = rotation*(*theVector);									//	apply the transformation to the vector
		theTile->position = rotation*theTile->position;						//	rotate the position of the tile
		theTile->normal = rotation*theTile->normal;							//	and its normal vector
		
		if (theVector->z > 0.01)	 										//	we don't want to divide by small values
			{
			zScale = viewScale*viewDistance/theVector->z;					//	scale based on the viewDistance
			thePoint->x = screenCentre.x + zScale*theVector->x;				//	map x & y directly
			thePoint->y = screenCentre.y + zScale*theVector->y;										
			if (insideTorus)												//	if we are inside the torus
				theTile->intensity = 256.0 * theTile->normal.z;				//	for a fixed perspective, this is true
			else
				theTile->intensity = -256.0 * theTile->normal.z;			//	for a fixed perspective, this is true
			} // end of case with non-zero z
		else theTile->intensity = 0;										//	avoids divide by zero errors

 		theVector++, theTile++, thePoint++;									//	advance our pointers
 		} // end of loop down arrays
		
	qsort((void *) sortedTiles, nTiles, sizeof(Tile **), compareTiles);		//	sort the tiles into order
	} // end of Rotate()

void Torus::TranslateZ(float z)												//	translate all points in z direction
	{
	Vector *theVector;														//	pointer walker
	Vector *stopVector;														//	sentinel value
	BPoint *thePoint;														//	pointer walker
	Tile *theTile;															//	tile walker
	float zScale;															//	amount to scale, based on z
	float r;																//	used to calculate whether we are
	Vector Origin;															//	the origin in torus-basis coordinates
	
	stopVector = theGridVectors + nTiles;									//	set sentinel
	theVector = theGridVectors;												//	start our walker
	thePoint = theGridPoints;												//	make that walker*s*
	theTile = theTiles;
	
	torusOrigin.z -= z;														//	move the origin of the torus

	Origin = torusBasis*torusOrigin;	

	r = sqrt(Origin.x*Origin.x+Origin.y*Origin.y);							//	calculate sqrt (x2 + y2)
	r = r - ringRadius;														//	subtract the ring radius
	r = sqrt(r*r + Origin.z*Origin.z);										//	finish the formula	
	insideTorus = (r <= radius);											//	determine if we're inside the torus

	for (; theVector < stopVector; theVector++, theTile++, thePoint++)		//	walk down three arrays in parallel
		{
		theVector->z -= z;													//	apply the translation to the vector
		theTile->position.z -= z;											//	translate the position of the tile
		if (theVector->z > 0.01)				 							//	otherwise, it's behind us
			{
			zScale = viewScale*viewDistance/theVector->z;					//	scale based on the viewDistance
			thePoint->x = screenCentre.x + zScale*theVector->x;				//	map x & y directly
			thePoint->y = screenCentre.y + zScale*theVector->y;										
			if (insideTorus)												//	if we are inside the torus
				theTile->intensity = 256.0 * theTile->normal.z;				//	for a fixed perspective, this is true
			else
				theTile->intensity = -256.0 * theTile->normal.z;			//	for a fixed perspective, this is true
			} // end of case with non-zero z
		else theTile->intensity = 0;										//	avoids divide by zero errors
		} // end of loop down arrays
		
	// 	note - no sort is required, since we sort on z, and all z's have been changed identically
	//	normals and intensity are unchanged, too
	} // end of TranslateZ()

bool Torus::RotateTiles(long direction)										//	rotates the contents of the tiles
	{
	Tile *stepTile, *bigStepTile, *stopTile, *bigStopTile;					//	pointers for conducting walks
	Tile *copyTile;															//	tile from which to copy
	
	bool swapIsSelected, swapHasPoint;										//	temporary storage for first tile's data
	SegmentNode *swapSegments;												//	ditto

	long i;																	//	loop index
	
	switch (direction)														//	depending on the direction passed in
		{
		case RotateUpItem:													//	the user wants to rotate up
			nPoints = 0;													//	set the number of points back to zero
			nPointsSelected = 0;											//	set the number of selected points back to zero
			bigStepTile = theTiles;											//	start off with the first tile
			bigStopTile = theTiles + nTiles;								//	and a suitable sentinel
			for ( ; bigStepTile < bigStopTile; bigStepTile += nPhiDivisions)//	walk around the ring
				{
				stepTile = bigStepTile;										//	start the walk around the tube
				stopTile = stepTile + nPhiDivisions - 1;					//	set our stop pointer to the last tile				
				swapIsSelected = stepTile->isSelected;						//	pick up the existing tile's vbles
				swapHasPoint = stepTile->hasPoint;
				swapSegments = stepTile->segmentList;				
				for ( ; stepTile < stopTile; stepTile++)					//	walk around the tube
					{
					copyTile = stepTile + 1;								//	we are copying from the next tile round
					stepTile->isSelected = copyTile->isSelected;			//	copy the items
					stepTile->hasPoint = copyTile->hasPoint;
					stepTile->segmentList = copyTile->segmentList;
					if (stepTile->hasPoint) 								//	if it has a point on it
						{
						thePoints[nPoints++] = stepTile;					//	put it on the list of points
						if (stepTile->isSelected) 							//	if it is selected
							selectedPoints[nPointsSelected++] = stepTile;	//	put it on the list of selected points
						} // end of test for points
					} // end of walk around tube
				stepTile->isSelected = swapIsSelected;						//	transfer the first tile's vbles back in
				stepTile->hasPoint = swapHasPoint;
				stepTile->segmentList = swapSegments;
				if (stepTile->hasPoint) 									//	if it has a point on it
					{
					thePoints[nPoints++] = stepTile;						//	put it on the list of points
					if (stepTile->isSelected) 								//	if it is selected
						selectedPoints[nPointsSelected++] = stepTile;		//	put it on the list of selected points
					} // end of test for points
				} // end of walk around ring	
			for (i = 0; i < nLines; i++)									//	adjust all the lines
				{
				theLines[i].phi--;											//	move it's coordinates down
				if (theLines[i].phi < 0)									//	if it fell off the bottom
					theLines[i].phi += nPhiDivisions;						//	adjust it upwards
				} //  end of loop to adjust lines
			break;
		case RotateDownItem:												//	the user wants to rotate down
			nPoints = 0;													//	set the number of points back to zero
			nPointsSelected = 0;											//	set the number of selected points back to zero
			bigStepTile = theTiles;											//	start off with the first tile
			bigStopTile = theTiles + nTiles;								//	and a suitable sentinel
			for ( ; bigStepTile < bigStopTile; bigStepTile += nPhiDivisions)//	walk around the ring
				{
				stopTile = bigStepTile;										//	start the walk around the tube
				stepTile = stopTile + nPhiDivisions - 1;					//	set our stop pointer to the last tile				
				swapIsSelected = stepTile->isSelected;						//	pick up the existing tile's vbles
				swapHasPoint = stepTile->hasPoint;
				swapSegments = stepTile->segmentList;				
				for ( ; stepTile > stopTile; stepTile--)					//	walk around the tube
					{
					copyTile = stepTile - 1;								//	we are copying from the next tile round
					stepTile->isSelected = copyTile->isSelected;			//	copy the items
					stepTile->hasPoint = copyTile->hasPoint;
					stepTile->segmentList = copyTile->segmentList;
					if (stepTile->hasPoint) 								//	if it has a point on it
						{
						thePoints[nPoints++] = stepTile;					//	put it on the list of points
						if (stepTile->isSelected) 							//	if it is selected
							selectedPoints[nPointsSelected++] = stepTile;	//	put it on the list of selected points
						} // end of test for points
					} // end of walk around tube
				stepTile->isSelected = swapIsSelected;						//	transfer the first tile's vbles back in
				stepTile->hasPoint = swapHasPoint;
				stepTile->segmentList = swapSegments;
				if (stepTile->hasPoint) 									//	if it has a point on it
					{
					thePoints[nPoints++] = stepTile;						//	put it on the list of points
					if (stepTile->isSelected) 								//	if it is selected
						selectedPoints[nPointsSelected++] = stepTile;		//	put it on the list of selected points
					} // end of test for points
				} // end of walk around ring	
			for (i = 0; i < nLines; i++)									//	adjust all the lines
				{
				theLines[i].phi++;											//	move it's coordinates up
				if (theLines[i].phi >= nPhiDivisions)						//	if it fell off the top
					theLines[i].phi -= nPhiDivisions;						//	adjust it downwards
				} //  end of loop to adjust lines
			break;
		case RotateLeftItem:												//	the user wants to rotate left
			nPoints = 0;													//	set the number of points back to zero
			nPointsSelected = 0;											//	set the number of selected points back to zero
			bigStepTile = theTiles;											//	start off with the first tile
			bigStopTile = theTiles + nPhiDivisions;							//	and a suitable sentinel
			for ( ; bigStepTile < bigStopTile; bigStepTile++)				//	walk around the tube
				{
				stopTile = bigStepTile;										//	start the walk around the ring
				stepTile = stopTile + nTiles - nPhiDivisions;				//	set our stop pointer to the last tile				
				swapIsSelected = stepTile->isSelected;						//	pick up the existing tile's vbles
				swapHasPoint = stepTile->hasPoint;
				swapSegments = stepTile->segmentList;				
				for ( ; stepTile > stopTile; stepTile -= nPhiDivisions)		//	walk around the tube
					{
					copyTile = stepTile - nPhiDivisions;					//	we are copying from the next tile round
					stepTile->isSelected = copyTile->isSelected;			//	copy the items
					stepTile->hasPoint = copyTile->hasPoint;
					stepTile->segmentList = copyTile->segmentList;
					if (stepTile->hasPoint) 								//	if it has a point on it
						{
						thePoints[nPoints++] = stepTile;					//	put it on the list of points
						if (stepTile->isSelected) 							//	if it is selected
							selectedPoints[nPointsSelected++] = stepTile;	//	put it on the list of selected points
						} // end of test for points
					} // end of walk around tube
				stepTile->isSelected = swapIsSelected;						//	transfer the first tile's vbles back in
				stepTile->hasPoint = swapHasPoint;
				stepTile->segmentList = swapSegments;
				if (stepTile->hasPoint) 									//	if it has a point on it
					{
					thePoints[nPoints++] = stepTile;						//	put it on the list of points
					if (stepTile->isSelected) 								//	if it is selected
						selectedPoints[nPointsSelected++] = stepTile;		//	put it on the list of selected points
					} // end of test for points
				} // end of walk around ring	
			for (i = 0; i < nLines; i++)									//	adjust all the lines
				{
				theLines[i].theta++;										//	move it's coordinates up
				if (theLines[i].theta >= nThetaDivisions)					//	if it fell off the top
					theLines[i].theta -= nThetaDivisions;					//	adjust it downwards
				} //  end of loop to adjust lines
			break;
		case RotateRightItem:												//	the user wants to rotate right
			nPoints = 0;													//	set the number of points back to zero
			nPointsSelected = 0;											//	set the number of selected points back to zero
			bigStepTile = theTiles;											//	start off with the first tile
			bigStopTile = theTiles + nPhiDivisions;							//	and a suitable sentinel
			for ( ; bigStepTile < bigStopTile; bigStepTile++)				//	walk around the tube
				{
				stepTile = bigStepTile;										//	start the walk around the ring
				stopTile = stepTile + nTiles - nPhiDivisions;				//	set our stop pointer to the last tile				
				swapIsSelected = stepTile->isSelected;						//	pick up the existing tile's vbles
				swapHasPoint = stepTile->hasPoint;
				swapSegments = stepTile->segmentList;				
				for ( ; stepTile < stopTile; stepTile += nPhiDivisions)		//	walk around the tube
					{
					copyTile = stepTile + nPhiDivisions;					//	we are copying from the next tile round
					stepTile->isSelected = copyTile->isSelected;			//	copy the items
					stepTile->hasPoint = copyTile->hasPoint;
					stepTile->segmentList = copyTile->segmentList;
					if (stepTile->hasPoint) 								//	if it has a point on it
						{
						thePoints[nPoints++] = stepTile;					//	put it on the list of points
						if (stepTile->isSelected) 							//	if it is selected
							selectedPoints[nPointsSelected++] = stepTile;	//	put it on the list of selected points
						} // end of test for points
					} // end of walk around tube
				stepTile->isSelected = swapIsSelected;						//	transfer the first tile's vbles back in
				stepTile->hasPoint = swapHasPoint;
				stepTile->segmentList = swapSegments;
				if (stepTile->hasPoint) 									//	if it has a point on it
					{
					thePoints[nPoints++] = stepTile;						//	put it on the list of points
					if (stepTile->isSelected) 								//	if it is selected
						selectedPoints[nPointsSelected++] = stepTile;		//	put it on the list of selected points
					} // end of test for points
				} // end of walk around ring	
			for (i = 0; i < nLines; i++)									//	adjust all the lines
				{
				theLines[i].theta--;										//	move it's coordinates down
				if (theLines[i].theta < 0)									//	if it fell off the bottom
					theLines[i].theta += nThetaDivisions;					//	adjust it upwards
				} //  end of loop to adjust lines
			break;
		default:															//	if it doesn't match one of the known rotations
			return false;													//	do nothing
		} // end of switch setting up rotation
	return true;															//	say that we changed things
	} // end of RotateTiles()