//	74.442	Advanced Design & Analysis of Algorithms
//	Spring 1998
//	Dr. Bill Kocay
//
//	Hamish Carr
//	Project:	Toroidal Surface Geometries
//
//

#include "TorusToolView.h"

#define paletteID 1000
#define paletteResourceWidth 71
#define paletteSquare 23
#define paletteFontSize 9
#define paletteLineHeight 12
#define paletteFirstLine 110
#define paletteIndent 2
#define paletteRows 4
#define paletteCols 3

TorusToolView::TorusToolView(BRect theFrame, Torus *newTorus)				//	constructor
	: BView(theFrame, "Torus Tool Palette", B_FOLLOW_LEFT+B_FOLLOW_TOP, B_WILL_DRAW)
																			//	call inherited constructor
	{
	BRect theRect;															//	for setting the size of pictures
	app_info CMAppInfo;														//	used for finding where the application is
	BEntry CMEntry;															//	entry in database representing the app
	BFile CMFile;															//	the CMass application file
	BResources CMResources;													//	the CMass resources
	void *aPICT;															//	a picture loaded - typed void b/c picH is a Mac type
	size_t pictLength;														//	length of the pict in bytes

	//	initialize array & rectangle

	theRect.Set(0, 0, paletteResourceWidth, paletteHeight);					//	set dimensions of the PICT
	theToolPaletteMap = NULL;												//	set the bitmap to be empty

	//	find the file in the database (no error checking)
	be_app->GetAppInfo(&CMAppInfo);											//	ask our global app to tell us where it is
	CMEntry.SetTo(&CMAppInfo.ref);											//	convert this to a database entry
	CMFile.SetTo(&CMEntry, B_READ_ONLY);									//	open it in read-only mode
	CMResources.SetTo(&CMFile);												//	open the resources

	aPICT = CMResources.FindResource('PICT', paletteID, &pictLength);		//	retrieve the picture
	theToolPaletteMap = new BBitmap(theRect, B_COLOR_8_BIT);				//	make a new bitmap with it
	theToolPaletteMap->SetBits(aPICT, pictLength, 0, B_COLOR_8_BIT);		//	and set the pixels
	free(aPICT);															//	free up the memory

	theTorus = newTorus;													//	set pointer to the torus
																			//	ask the app to retrieve the bitmap	
	SetFontSize(paletteFontSize);											//	set the font size
	} // end of TorusToolView constructor

void TorusToolView::Draw(BRect updateRect)									//	draw the torus
	{
	#pragma unused(updateRect)
	long row, col;															//	the row & column of the tool
	BRect theRect;															//	for inverting chosen tool
	char textBuffer[256];													//	buffer for printing text
	float xPosn, yPosn;														//	position of next item to be drawn
	BPicture *collectionBox;												//	picture to buffer drawing calls
	Vector viewPoint;														//	the view point relative to the torus origin
		
	BeginPicture(new BPicture);												//	start capturing calls

	FillRect(Bounds(), B_SOLID_LOW);										//	clear the rectangle

	row = theTorus->theCurrentTool/3;										//	find which row
	col = theTorus->theCurrentTool%3;										//	and which column
	
	MovePenTo(0,0);															//	move the pen into place
	DrawBitmap(theToolPaletteMap);											//	draw the palette

	theRect.Set(1, 1, paletteSquare-1, paletteSquare-1);					//	set the size of the chosen tool			
	theRect.OffsetBy(col*paletteSquare, row*paletteSquare);					//	then the location
	InvertRect(theRect);													//	and invert it
	
	MovePenTo(paletteWidth-1, 0);											//	move to righthand edge
	StrokeLine(BPoint(paletteWidth-1, Bounds().Height()));					//	draw vertical line

	xPosn = paletteIndent;
	yPosn = paletteFirstLine;												//	set the initial position

	MovePenTo(xPosn, yPosn);												//	move pen, ready to draw
	sprintf(textBuffer, "Mesh:");
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen, ready to draw
	sprintf(textBuffer, "     %d x %d", theTorus->nThetaDivisions, theTorus->nPhiDivisions);
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "Ring Radius:");									//	set up the string
	DrawString(textBuffer);													//	draw the string

	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "     %0.2f", theTorus->ringRadius);				//	set up the string
	DrawString(textBuffer);													//	draw the string

	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "Tube Radius:");									//	set up the string
	DrawString(textBuffer);													//	draw the string

	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "     %0.2f", theTorus->radius);					//	set up the string
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "Points:");											//	set up the string
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "     %d", theTorus->nPoints);						//	set up the string
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "Selected:");										//	set up the string
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "     %d", theTorus->nPointsSelected);				//	set up the string
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "Lines:");											//	set up the string
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "     %d", theTorus->nLines);						//	set up the string
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "Selected:");										//	set up the string
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "     %d", theTorus->nLinesSelected);				//	set up the string
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "Scale:");											//	set up the string
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "     %0.2f", theTorus->viewScale);					//	set up the string
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "Distance:");										//	set up the string
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "     %.0f", theTorus->viewDistance);				//	set up the string
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "ViewPoint:");
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	viewPoint = theTorus->torusBasis.k;										//	start off looking at the origin
	viewPoint = theTorus->torusOrigin.z *  viewPoint;						//	and scale it
	sprintf(textBuffer, "     (%.0f, %.0f, %.0f)", viewPoint.x, viewPoint.y, viewPoint.z);
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "View Facing:");
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "     (%.0f, %.0f, %.0f)", -100*(theTorus->torusBasis.k.x), -100*(theTorus->torusBasis.k.y), -100*(theTorus->torusBasis.k.z));
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "View Azimuth:");
	DrawString(textBuffer);													//	draw the string
	
	yPosn +=paletteLineHeight;												//	move to next logical line
	MovePenTo(xPosn, yPosn);												//	move pen
	sprintf(textBuffer, "     (%.0f, %.0f, %.0f)", 100*theTorus->torusBasis.j.x, 100*theTorus->torusBasis.j.y, 100*theTorus->torusBasis.j.z);
	DrawString(textBuffer);													//	draw the string

	collectionBox = EndPicture();											//	stop capturing calls
	MovePenTo(0,0);															//	return to top left for drawing
	DrawPicture(collectionBox);												//	and draw the picture properly
	delete collectionBox;													//	get rid of the picture
	} // end of Draw()
	
TorusToolView::~TorusToolView()												//	destructor
	{
	delete theToolPaletteMap;												//	delete the bitmap of the palette
	} // end of TorusToolView()

void TorusToolView::MouseDown(BPoint where)									//	handle a mouse click
	{
	long row, col;															//	which row, column of palette was hit
	long whichTool;															//	which tool was hit
	
	row = where.y/paletteSquare;											//	calculate which row was clicked
	if (where.y == row * paletteSquare) return;								//	ignore clicks on palette lines
	if (row >= paletteRows) return;											//	likewise for clicks off palette proper
	
	col = where.x/paletteSquare;											//	calculate which column
	if (where.x == col * paletteSquare) return;								//	ignore clicks on palette lines
	if (col >= paletteCols) return;											//	and ignore clicks off the side	
	
	whichTool = col + (row * paletteCols);									//	compute the tool ID
	
	switch (whichTool)														//	what to do depends on the tool
		{
		case selectionTool:													//	selects a point
			break;
		case pointTool:														//	creates a new point
			break;
		case lineTool:														//	creates a new line
			break;
		case handTool:														//	moves the entire object
			break;
		case connectTool:													//	connects 2 or more selected points
			Window()->PostMessage(ConnectItem, theTorusView);				//	pass the request on as a message
			return;
		case intersectTool:													//	intersects 2 or more selected lines
			Window()->PostMessage(IntersectItem, theTorusView);				//	pass the request on as a message
			return;
		case rotateUpTool:													//	rotates outside face up round tube
			Window()->PostMessage(RotateUpItem, theTorusView);				//	pass the request on as a message
			return;
		case rotateDownTool:												//	rotates outside face down round tube
			Window()->PostMessage(RotateDownItem, theTorusView);			//	pass the request on as a message
			return;
		case rotateLeftTool:												//	rotates points left round torus
			Window()->PostMessage(RotateLeftItem, theTorusView);			//	pass the request on as a message
			return;
		case rotateRightTool:												//	rotates points right round torus
			Window()->PostMessage(RotateRightItem, theTorusView);			//	pass the request on as a message
			return;
		default:															//	all other cases do nothing
			return;															//	so just bail out
		} // end of cases for various tools
	theTorus->theCurrentTool = whichTool;									//	reset the current tool
	Draw(Bounds());															//	redraw the entire tool palette
	} // end of MouseDown()