//	74.442	Advanced Design & Analysis of Algorithms
//	Spring 1998
//	Dr. Bill Kocay
//
//	Hamish Carr
//	Project:	Toroidal Surface Geometries
//
//

#include "TorusWindow.h"

char *otherText = "&Torus\ntorus1\nTube Radius:\n1000.000000\nRing Radius:\n2000.000000\n# Phi Divisions:\n24\n# Theta Divisions\n120\n# Points\n4\n33 19\n28 20\n28 18\n17 19\n# Lines\n3\n28 18\n5 1\n28 18\n0 1\n33 19\n-5 1\n";

TorusWindow::TorusWindow(BRect theFrame, char *theTorusText, BMessage *theLocation)
	: BWindow(theFrame, "Torus", B_TITLED_WINDOW, B_NOT_RESIZABLE)			//	call inherited constructor
	{
	BRect theRect;															//	used for setting views
	BPoint theCentre;														//	the centre of the window

	theRect.Set(0, 0, theFrame.Width(), 18);								//	set the size of the menu bar
	theMenuBar = new TorusMenuBar(theRect);									//	create the menu bar
	AddChild(theMenuBar);													//	and add it to the window

	theRect.Set(paletteWidth, 18, theFrame.Width(), theFrame.Height());
																			//	set the size of the rendering view

	if (theTorusText == NULL)												//	i.e. this is a brand new torus
		{
		theTorus = new Torus();												//	create the torus
		} // end of case where entirely new torus
	else
		{
		theTorus = new Torus(theTorusText, theLocation);					//	create it from saved versions
		} // end of case where torus was saved

	Basis theInitialBasis(Vector(1, 0, 0), Vector(0, 0.8, 0.6), Vector(0, -0.6, 0.8));
																			//	the initial basis for viewing the torus
	theCentre = BPoint(theRect.Width()/2, theRect.Height()/2);				//	set the centre
	theTorus->SetRenderingParms(theCentre, 5000, 0.05);						//	initialize the rendering parameters
	theTorus->Rotate(theInitialBasis);										//	rotate the torus to initial viewing angle
	theTorus->TranslateZ(-5000);											//	move it back so we can see

	theTorusView = new TorusView(theRect, theTorus);						//	and create the torus view
	AddChild(theTorusView);													//	then add it to the application

	theRect.Set(0, 18, paletteWidth-1, theFrame.Height());					//	create the rectangle for the tools
	theTorusToolView = new TorusToolView(theRect, theTorus);				//	create the tool view
	AddChild(theTorusToolView);												//	then add it to the application

	theTorusToolView->theTorusView = theTorusView;							//	tell the tool view about the main view
	theTorusView->theTorusToolView = theTorusToolView;						//	tell the torus view about the tools
	theTorusView->MakeFocus(true);											//	set it to receive keyboard events
		
	itsMessenger = new BMessenger(this);									//	set the messenger to call the window
	itsSavePanel = new BFilePanel(B_SAVE_PANEL, itsMessenger, NULL, 0, false);
																			//	set up a file panel for saving
	pageSetup = NULL;														//	set this to something safe
	} // end of TorusWindow() constructor

TorusWindow::~TorusWindow()													//	destructor
	{
	delete itsSavePanel;													//	get rid of the associated save panel
	delete itsMessenger;													//	and the messenger
	if (pageSetup == NULL) delete pageSetup;								//	and the page setup message
	delete theTorus;														//	make sure that the torus is destroyed, too
	} // end of TorusWindow destructor

bool TorusWindow::QuitRequested()											//	our exit routine
	{
	long nWindows;															//	how many windows the app has
	nWindows = be_app->CountWindows();										//	find out how many
	if (nWindows <= 4)														//	i.e. this is the last window other than the save panel
																			//	and the new torus panel (remember the offscreen blitter)
		be_app->PostMessage(B_QUIT_REQUESTED);								//	tell the application to quit
	return(true);															//	and exit
	} // end of QuitRequested()

void TorusWindow::MessageReceived(BMessage *theEvent)						//	processes messages
	{
	switch(theEvent->what)													//	yer basic case statement
		{
		case AboutTorusItem:												//	user has asked for the about box
		case NewTorusItem:													//	user has asked for another torus
		case OpenTorusItem:													//	user has asked to open a file on disk
			be_app->PostMessage(theEvent);									//	forward the event to the application
			break;
		case SaveTorusItem:													//	user has asked to save the torus
			if (theTorus->itsLocation == NULL)								//	if the torus is so far anonymous
				ShowSavePanel();											//	show the save panel
			else
				SaveAsText(theTorus->itsLocation);							//	otherwise, save, using existing location
			break;
		case SaveAsTorusItem:												//	user has asked to save the torus
			ShowSavePanel();												//	show the save panel
			break;
		case B_SAVE_REQUESTED :												// 	put it here b/c it's in response to the save
			SaveAsText(theEvent);											// 	call the save routine (& pass the event for it's fields)
			break;
		case CloseTorusItem:												//	user has asked to close this item
			Looper()->PostMessage(B_QUIT_REQUESTED);						//	request that the looper/window quit
			break;	
		case PageSetupItem:													//	user has asked to set up page for printing
			PageSetup();													//	so do it
			break;
		case PrintItem:														//	user has asked to print
			Print();														//	so set it to print out
			break;
		case QuitItem:														//	user wants to quit
			be_app->PostMessage(B_QUIT_REQUESTED);							//	request that the application quit
			break;
		default:
			BWindow::MessageReceived(theEvent);								//	pass it on to the BWindow class
			break;
		} // end of switch statement	
	} // end of MessageReceived
	
void TorusWindow::ShowSavePanel()											//	shows the save panel for this file
	{
	itsSavePanel->SetSaveText(theTorus->name);								//	set a default name for the save as
	itsSavePanel->Show();													//	show the panel	
	} // end of ShowSavePanel()

void TorusWindow::SaveAsText(BMessage *theEvent)							//	routine to save a file in text format
	{
	status_t success;														//	used for seeing if system calls worked
	entry_ref theRef;														// 	in which directory the file should be saved
	char *theName; 															// 	what name to give the file (already passed check for name collision)
	BFile theFile; 															// 	the file to which to save
	BDirectory theDirectory; 												// 	the directory in which to save
	char *walk;																//	pointer for walking along proposed name
	char *torusText;														//	text representation of torus to save
	long textLength;														//	how many bytes the text takes
	
	success = theEvent->FindRef("directory", &theRef);						// 	get the directory ref from the message
	if (success != B_NO_ERROR)												// 	if that didn't work
		{
		DoAlert("Unable to locate directory for saving file.");				//	pop up an alert box
		return;										
		} // end of test for existence of directory

	success = theDirectory.SetTo(&theRef);									// 	try to convert the ref to a directory
	if (success != B_OK)												 	// 	if that didn't work
		{
		DoAlert("Unable to open directory.\n");								// 	complain
		return;
		} // end of attempt to open directory

	success = theEvent->FindString("name", &theName);						// 	get the name of the file from the message
	if (success != B_NO_ERROR)												// 	if that didn't work
		{
		theDirectory.Unset();												//	get rid of the directory
		DoAlert("Name not found in message.\n");							// 	Likewise complain
		return;
		} // end of test for directory

	for (walk = theName; *walk != '\0'; walk++)								// 	walk along the string with the name
		if (*walk == '/')													// 	if it's the forbidden directory separator
			*walk = '_';													// 	replace with an innocuous underscore	

	success = theDirectory.CreateFile(theName, &theFile, false);			// 	create the file, overwriting if necessary
	if (success != B_NO_ERROR)												// 	if that failed
		{
		DoAlert("Unable to create file.\n");								// 	tell the user
		theDirectory.Unset();												//	get rid of the directory
		delete theName;														// 	get rid of dynamic storage
		return;
		} // end of attempt to create file
	
	if (theTorus->itsLocation != NULL)										//	i.e if the torus was already saved
		delete theTorus->name;												//	release the storage for it's name
	theTorus->name = theName;												//	set the name of the torus

	torusText = theTorus->MakeText();										//	create the text representation
	if (torusText == NULL)													//	if that failed
		{
		DoAlert("Unable to create text.\n");								// 	tell the user
		theDirectory.Unset();												//	get rid of the directory
		theFile.Unset();													//	and get rid of the file
		delete theName;														// 	get rid of dynamic storage
		return;
		} // end of attempt to construct text
			
	textLength = strlen(torusText);											// 	find out how big it is
	success = theFile.SetSize(textLength);									// 	try to make sure that there is enough space
	if (success != B_NO_ERROR)												// 	if it failed
		{
		DoAlert("Unable to allocate space for file.\n");					//	tell the user
		}// end of failure to set size
	else																	//	i.e. space has been allocated
		{
		success = theFile.Write(torusText, textLength);						// 	attempt to write it to file
		if (success < B_NO_ERROR)											// 	if it failed
			DoAlert("Unable to write torus into file.\n");					//	tell the user
		else
			theTorus->itsLocation = theEvent;								//	store the message representing disk location
		} // end of successful allocation
	theFile.Unset();														//	clear up the dynamic structures
	theDirectory.Unset();													//	both directory and file
	delete torusText;														// 	delete the graph itself
	} // end of SaveAsText()
	
void TorusWindow::PageSetup()												//	sets up the page for printing
{
BPrintJob job("Torus Print");												// 	initialize a print job
status_t errorCode;															//	for catching return values

if (pageSetup != NULL)														//	if the message already exists
	job.SetSettings(new BMessage(pageSetup));								// 	copy it for setting up the job
errorCode = job.ConfigPage();												// 	call the dialog to configure the page
if (errorCode == B_OK)														// 	if we succeeded
	{
	delete pageSetup;														// 	delete the old config message
	pageSetup = job.Settings();												// 	and save the new config message
	}; // end of if errorCode
} // end of PageSetup()

void TorusWindow::Print()													// 	handles request to print curve
{
BPrintJob job("Torus Print");												// 	initialize a print job with the name of the graph
status_t errorCode;															// 	for catching return values
BPoint printOrigin;															// 	origin to pass to DrawView() for printing

// set up the page for printing

if (pageSetup != NULL)														// 	if the message already exists
	job.SetSettings(new BMessage(pageSetup));								// 	copy it for setting up the job
else // i.e. there is no stored setup
	{
	errorCode = job.ConfigPage();											// 	call the dialog to configure the page
	if (errorCode == B_OK)													// 	if we succeeded
		{
		delete pageSetup;													// 	delete the old config message
		pageSetup = job.Settings();											// 	and save the new config message
		}; // end of if errorCode
	} // end of case of no setup message
errorCode = job.ConfigJob();												// 	call the dialog to configure which pages, &c.
if (errorCode != B_OK)														// 	if we can't configure the job
	return;																	// 	bail out of the print function

// start the print job	
job.BeginJob();																// 	start recording drawing calls
		
job.DrawView(theTorusView, theTorusView->Bounds(), theTorusView->Frame().LeftTop());
																			// 	draw the curve proper in the right place
job.SpoolPage();															// 	and spool the page off to be printed

// send the print job off to be printed
if  (job.CanContinue())														// 	if we haven't met a just cause or impediment. . .
	job.CommitJob();														//	commit the whole job to be printed
else	// i.e. we can't continue for some reason					
	job.CancelJob();														// 	cancel the job in it's entirety
} // end of Print()
